/*
    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"
#include "auth.h"
#include "master.h"

typedef struct {
	Client *client;
	char *user;
        LoginCallback callback;
} Request;

static void login_callback(int success, void *user_data)
{
	Request *req = user_data;

	if (client_unref(req->client))
		req->callback(req->client, req->user, success);

	i_free(req->user);
	i_free(req);
}

static void login(Client *client, const char *user, const char *pass,
		  LoginCallback callback)
{
	Request *req;

	req = i_new(Request, 1);
	req->client = client;
	req->user = i_strdup(user);
	req->callback = callback;

	client_ref(client);
	master_send_login(user, pass, client->fd, client->tag,
			  login_callback, req);
}

AuthModule auth_master = {
	"master",

	NULL, /* init */
	NULL, /* deinit */

	login,
	NULL
};
